@@ -22,6 +22,7 @@ import traceback
 from PyQt4 import QtCore
 from picard.metadata import Metadata, run_album_metadata_processors, run_track_metadata_processors
 from picard.dataobj import DataObject
+from picard.file import File
 from picard.track import Track
 from picard.script import ScriptParser
 from picard.ui.item import Item
@@ -33,9 +34,16 @@ from picard.mbxml import release_to_metadata, track_to_metadata
 VARIOUS_ARTISTS_ID = '89ad4ac3-39f7-470e-963a-56509c546377'
 
 
+_TRANSLATE_TAGS = {
+    "hip hop": u"Hip-Hop",
+    "synth-pop": u"Synthpop",
+    "electronica": u"Electronic",
+}
+
+
 class ReleaseEvent(object):
 
-    ATTRS = ['date', 'releasecountry', 'label', 'barcode', 'catalognumber']
+    ATTRS = ['date', 'releasecountry', 'label', 'barcode', 'catalognumber', 'format']
 
     def __init__(self):
         for attr in self.ATTRS:
@@ -50,13 +58,40 @@ class ReleaseEvent(object):
                 try: del m[attr]
                 except KeyError: pass
 
+    def from_metadata(self, m):
+        for attr in self.ATTRS:
+           if m[attr]: setattr(self, attr, m[attr])
+
+    def similarity(self, m):
+        sim = 0.0
+        if not m:
+            return sim
+        for attr in self.ATTRS:
+           val = getattr(self, attr)
+           mval = getattr(m, attr)
+           if val and mval:
+               if attr == 'date':
+                   dsim = 0.0
+                   sdate = val.split('-') 
+                   mdate = mval.split('-')
+                   for i in range(min(len(sdate),len(mdate))):
+                      if sdate[i] == mdate[i]:
+                         dsim+=1.0       
+                      else:
+                         break
+                   dsim/=max(len(mdate),len(sdate))
+                   sim+=dsim
+               else:
+                   if mval == val:
+                       sim+=1.0
+        sim/=len(self.ATTRS)
+        return sim
 
 class Album(DataObject, Item):
 
     def __init__(self, id, catalognumber=None):
         DataObject.__init__(self, id)
         self.metadata = Metadata()
-        self.unmatched_files = Cluster(_("Unmatched Files"), special=2)
         self.tracks = []
         self.loaded = False
         self._files = 0
@@ -64,6 +99,7 @@ class Album(DataObject, Item):
         self._catalognumber = catalognumber
         self.current_release_event = None
         self.release_events = []
+        self.unmatched_files = Cluster(_("Unmatched Files"), special=2, related_album=self)
 
     def __repr__(self):
         return '<Album %s %r>' % (self.id, self.metadata[u"album"])
@@ -76,6 +112,36 @@ class Album(DataObject, Item):
             for file in self.unmatched_files.iterfiles():
                 yield file
 
+    def _convert_folksonomy_tags_to_genre(self, track, ignore_tags):
+        # Combine release and track tags
+        tags = dict(self.folksonomy_tags)
+        for name, count in track.folksonomy_tags:
+            tags.setdefault(name, 0)
+            tags[name] += count
+        if not tags:
+            return
+        # Convert counts to values from 0 to 100
+        maxcount = max(tags.values())
+        taglist = []
+        for name, count in tags.items():
+            taglist.append((100 * maxcount / count, name))
+        taglist.sort(reverse=True)
+        # And generate the genre metadata tag
+        maxtags = self.config.setting['max_tags']
+        minusage = self.config.setting['min_tag_usage']
+        genre = []
+        for usage, name in taglist[:maxtags]:
+            if name in ignore_tags:
+                continue
+            if usage < minusage:
+                break
+            name = _TRANSLATE_TAGS.get(name, name.title())
+            genre.append(name)
+        join_tags = self.config.setting['join_tags']
+        if join_tags:
+            genre = [join_tags.join(genre)]
+        track.metadata['genre'] = genre
+
     def _parse_release(self, document):
         self.log.debug("Loading release %r", self.id)
 
@@ -93,7 +159,12 @@ class Album(DataObject, Item):
                 break
         else:
             if self.release_events:
-                self.current_release_event = self.release_events[0]
+                preferred_events = [rel for rel in self.release_events
+                                    if rel.releasecountry == self.config.setting["preferred_release_country"]]
+                if preferred_events:
+                    self.current_release_event = preferred_events[0]
+                else:
+                    self.current_release_event = self.release_events[0]
         if self.current_release_event:
             self.current_release_event.to_metadata(m)
 
@@ -121,6 +192,8 @@ class Album(DataObject, Item):
         # Strip leading/trailing whitespace
         m.strip_whitespace()
 
+        ignore_tags = [s.strip() for s in self.config.setting['ignore_tags'].split(',')]
+
         artists = set()
         for i, node in enumerate(release_node.track_list[0].track):
             t = Track(node.attribs['id'], self)
@@ -130,7 +203,7 @@ class Album(DataObject, Item):
             tm = t.metadata
             tm.copy(m)
             tm['tracknumber'] = str(i + 1)
-            track_to_metadata(node, tm, config=self.config)
+            track_to_metadata(node, tm, config=self.config, track=t)
             artists.add(tm['musicbrainz_artistid'])
             m.length += tm.length
 
@@ -142,7 +215,10 @@ class Album(DataObject, Item):
             if tm['musicbrainz_artistid'] == VARIOUS_ARTISTS_ID:
                 tm['artistsort'] = tm['artist'] = self.config.setting['va_name']
 
-            # Album metadata plugins
+            if self.config.setting['folksonomy_tags']:
+                self._convert_folksonomy_tags_to_genre(t, ignore_tags)
+
+            # Track metadata plugins
             try:
                 run_track_metadata_processors(self, tm, release_node, node)
             except:
@@ -186,19 +262,24 @@ class Album(DataObject, Item):
         else:
             if not self._requests:
                 for old_track, new_track in zip(self.tracks, self._new_tracks):
-                    if old_track.linked_file:
-                        new_track.linked_file = old_track.linked_file
-                        new_track.linked_file.parent = new_track
-                        new_track.linked_file.metadata.copy(new_track.metadata)
-                        new_track.linked_file.update(signal=False)
+                    for file in old_track.linked_files:
+                        new_track.linked_files.append(file)
+                        file.parent = new_track
+                        file.metadata.copy(new_track.metadata)
+                        file.update(signal=False)
                 for track in self.tracks[len(self._new_tracks):]:
-                    if track.linked_file:
-                        track.linked_file.move(self.unmatched_files)
+                    for file in track.linked_files:
+                        file.move(self.unmatched_files)
                 self.metadata = self._new_metadata
                 self.tracks = self._new_tracks
                 del self._new_metadata
                 del self._new_tracks
                 self.loaded = True
+                for track in self.tracks:
+                    for file in track.linked_files:
+                        if file.orig_metadata:
+                            self.match_release_event(file.orig_metadata)
+                            break
                 self.update()
                 self.tagger.window.set_statusbar_message('Album %s loaded', self.id, timeout=3000)
                 self.match_files(self.unmatched_files.files)
@@ -215,13 +296,13 @@ class Album(DataObject, Item):
         self._new_metadata = Metadata()
         self._new_tracks = []
         self._requests = 1
+        inc = ['tracks', 'puids', 'artist', 'release-events', 'labels']
         if self.config.setting['release_ars'] or self.config.setting['track_ars']:
+            inc += ['artist-rels', 'url-rels']
             if self.config.setting['track_ars']:
-                inc = ('tracks', 'puids', 'artist', 'release-events', 'labels', 'artist-rels', 'url-rels', 'track-level-rels')
-            else:
-                inc = ('tracks', 'puids', 'artist', 'release-events', 'labels', 'artist-rels', 'url-rels')
-        else:
-            inc = ('tracks', 'puids', 'artist', 'release-events')
+                inc += ['track-level-rels']
+        if self.config.setting['folksonomy_tags']:
+            inc += ['tags']
         self.tagger.xmlws.get_release_by_id(self.id, self._release_request_finished, inc=inc)
 
     def update(self, update_tracks=True):
@@ -229,6 +310,8 @@ class Album(DataObject, Item):
 
     def _add_file(self, track, file):
         self._files += 1
+        if file.orig_metadata:
+            self.match_release_event(file.orig_metadata)
         self.update(False)
 
     def _remove_file(self, track, file):
@@ -257,9 +340,7 @@ class Album(DataObject, Item):
         for sim, file, track in matches:
             if sim < self.config.setting['track_matching_threshold']:
                 break
-            if file in matched or track in matched.values():
-                continue
-            if track.linked_file and sim < track.linked_file.similarity:
+            if file in matched:
                 continue
             matched[file] = track
         unmatched = [f for f in files if f not in matched]
@@ -296,13 +377,21 @@ class Album(DataObject, Item):
         return len(self.unmatched_files.files)
 
     def get_num_unsaved_files(self):
-        return len([track for track in self.tracks
-                    if track.linked_file and not track.linked_file.is_saved()])
+        count = 0
+        for track in self.tracks:
+            for file in track.linked_files:
+                if not file.is_saved():
+                    count+=1
+        return count
 
     def column(self, column):
         if column == 'title':
             if self.tracks:
-                text = u'%s\u200E (%d/%d' % (self.metadata['album'], self._files, len(self.tracks))
+                linked_tracks = 0
+                for track in self.tracks:
+                    if track.is_linked():
+                        linked_tracks+=1
+                text = u'%s\u200E (%d/%d' % (self.metadata['album'], linked_tracks, len(self.tracks))
                 unmatched = self.get_num_unmatched_files()
                 if unmatched:
                     text += '; %d?' % (unmatched,)
@@ -330,18 +419,40 @@ class Album(DataObject, Item):
             self.update(update_tracks=False)
             for track in self.tracks:
                 self.current_release_event.to_metadata(track.metadata)
-                if track.linked_file:
-                    self.current_release_event.to_metadata(track.linked_file.metadata)
-                    track.linked_file.update()
-                else:
+                for file in track.linked_files:
+                    self.current_release_event.to_metadata(file.metadata)
+                    file.update()
+                if len(track.linked_files) <> 1:
                     track.update()
 
-    def add_release_event(self, date=None, releasecountry=None, label=None, barcode=None, catalognumber=None):
+    def add_release_event(self, date=None, releasecountry=None, label=None, barcode=None, catalognumber=None, format=None):
         rel = ReleaseEvent()
         rel.date = date
         rel.releasecountry = releasecountry
         rel.label = label
         rel.barcode = barcode
         rel.catalognumber = catalognumber
+        rel.format = format
         self.release_events.append(rel)
         return rel
+
+    def match_release_event(self, obj):
+        rel = ReleaseEvent()
+        if isinstance(obj, ReleaseEvent):
+            rel = obj
+        elif isinstance(obj, Metadata):
+            rel.from_metadata(obj)
+        elif isinstance(obj, File):
+            if obj.metadata:
+                rel.from_metadata(obj.metadata)
+        else:
+            self.log.error("Unsupported type given")
+            return
+
+        matches = []
+        if self.release_events:
+            for rrel in self.release_events:
+                sim = rrel.similarity(rel)
+                matches.append((sim, rrel))
+            matches.sort(reverse=True)
+            if matches[0] and matches[0][0] > 0: self.set_current_release_event(matches[0][1])
