@@ -86,10 +86,10 @@ class File(LockableObject, Item):
         return '<File #%d %r>' % (self.id, self.base_filename)
 
     def load(self, next, thread_pool):
-        thread_pool.call(thread_pool.LOAD,
-                         partial(self._load, self.filename),
-                         partial(self._loading_finished, next),
-                         QtCore.Qt.LowEventPriority + 1)
+        self.tagger.load_queue.put((
+            partial(self._load, self.filename),
+            partial(self._loading_finished, next),
+            QtCore.Qt.LowEventPriority + 1))
 
     @call_next
     def _loading_finished(self, next, result=None, error=None):
@@ -133,11 +133,10 @@ class File(LockableObject, Item):
         metadata = Metadata()
         metadata.copy(self.metadata)
         metadata.strip_whitespace()
-        thread_pool.call(
-            thread_pool.SAVE,
+        self.tagger.load_queue.put((
             partial(self._save_and_rename, self.filename, metadata, settings),
             partial(self._saving_finished, next),
-            QtCore.Qt.LowEventPriority + 2)
+            QtCore.Qt.LowEventPriority + 2))
 
     def _save_and_rename(self, old_filename, metadata, settings):
         """Save the metadata."""
@@ -186,19 +185,19 @@ class File(LockableObject, Item):
     def _script_to_filename(self, format, file_metadata, settings):
         metadata = Metadata()
         metadata.copy(file_metadata)
-        # replace incompatible characters
+        # make sure every metadata can safely be used in a path name
         for name in metadata.keys():
-            value = metadata[name]
-            if isinstance(value, basestring):
-                value = sanitize_filename(value)
-                if settings["windows_compatible_filenames"] or sys.platform == "win32":
-                    value = replace_win32_incompat(value)
-                if settings["ascii_filenames"]:
-                    if isinstance(value, unicode):
-                        value = unaccent(value)
-                    value = replace_non_ascii(value)
-                metadata[name] = value
-        return ScriptParser().eval(format, metadata)
+            if isinstance(metadata[name], basestring):
+                metadata[name] = sanitize_filename(metadata[name])
+        filename = ScriptParser().eval(format, metadata)
+        # replace incompatible characters
+        if settings["windows_compatible_filenames"] or sys.platform == "win32":
+            filename = replace_win32_incompat(filename)
+        if settings["ascii_filenames"]:
+            if isinstance(filename, unicode):
+                filename = unaccent(filename)
+            filename = replace_non_ascii(filename)
+        return filename
 
     def _make_filename(self, filename, metadata, settings):
         """Constructs file name based on metadata and file naming formats."""
@@ -397,10 +396,14 @@ class File(LockableObject, Item):
     state = property(get_state, set_state)
 
     def column(self, column):
-        if column == '~length':
-            return format_time(self.metadata.length), self.similarity
+        if self.orig_metadata:
+            md = self.orig_metadata
         else:
-            return self.metadata[column], self.similarity
+            md = self.metadata
+        if column == '~length':
+            return format_time(md.length), self.similarity
+        else:
+            return md[column], self.similarity
 
     def _compare_to_track(self, track):
         """
@@ -485,7 +488,7 @@ class File(LockableObject, Item):
             threshold = self.config.setting['file_lookup_threshold']
 
         if matches[0][0] < threshold:
-            self.tagger.window.set_statusbar_message(N_("No matching tracks for file %s"), self.filename, timeout=3000)
+            self.tagger.window.set_statusbar_message(N_("No matching tracks above the threshold for file %s"), self.filename, timeout=3000)
             self.clear_pending()
             return
         self.tagger.window.set_statusbar_message(N_("File %s identified!"), self.filename, timeout=3000)
@@ -512,7 +515,7 @@ class File(LockableObject, Item):
             tnum=self.metadata.get('tracknumber', ''),
             tracks=self.metadata.get('totaltracks', ''),
             qdur=str(self.metadata.length / 2000),
-            limit=7)
+            limit=25)
 
     def set_pending(self):
         if self.state == File.REMOVED:
