@@ -78,11 +78,11 @@ from picard.util import (
     webbrowser2,
     pathcmp,
     partial,
+    queue,
+    thread
     )
-from picard.util.thread import ThreadPool
 from picard.webservice import XmlWebService
 
-
 class Tagger(QtGui.QApplication):
 
     """
@@ -126,7 +126,29 @@ class Tagger(QtGui.QApplication):
         self.cachedir = os.path.join(self.userdir, "cache")
 
         # Initialize threading and allocate threads
-        self.thread_pool = ThreadPool(self)
+        self.thread_pool = thread.ThreadPool(self)
+        
+        self.load_queue = queue.Queue()
+        self.load_queue.run_item = thread.generic_run_item
+
+        self.save_queue = queue.Queue()
+        self.save_queue.run_item = thread.generic_run_item
+
+        self.analyze_queue = queue.Queue()
+        self.analyze_queue.run_item = analyze_thread_run_item
+        self.analyze_queue.next = self._lookup_puid
+
+        self.other_queue = queue.Queue()
+        self.other_queue.run_item = thread.generic_run_item
+        
+        threads = self.thread_pool.threads
+        threads.append(thread.Thread(self.thread_pool, [self.load_queue,
+                                                        self.other_queue]))
+        threads.append(thread.Thread(self.thread_pool, [self.save_queue]))
+        threads.append(thread.Thread(self.thread_pool, [self.other_queue,
+                                                        self.load_queue]))
+        threads.append(thread.Thread(self.thread_pool, [self.analyze_queue]))
+        
         self.thread_pool.start()
         self.stopping = False
 
@@ -161,6 +183,7 @@ class Tagger(QtGui.QApplication):
         # Initialize fingerprinting
         self._ofa = musicdns.OFA()
         self._ofa.init()
+        self.analyze_queue.ofa = self._ofa
 
         # Load plugins
         self.pluginmanager = PluginManager()
@@ -319,33 +342,40 @@ class Tagger(QtGui.QApplication):
             if result is not None and error is None:
                 files = []
                 directories = []
-                for path in result:
-                    path = os.path.join(root, path)
-                    if os.path.isdir(path):
-                        directories.append(path)
-                    else:
-                        files.append(decode_filename(path))
-                if files:
-                    self.add_files(files)
-                delay = 25 * len(files)
-                queue = directories + queue
+                try:
+                    for path in result:
+                        path = os.path.join(root, path)
+                        if os.path.isdir(path):
+                            directories.append(path)
+                        else:
+                            try:
+                                files.append(decode_filename(path))
+                            except UnicodeDecodeError:
+                                self.log.warning("Failed to decode filename: %r", path)
+                                continue
+                finally:
+                    if files:
+                        self.add_files(files)
+                    delay = min(25 * len(files), 500)
+                    queue = directories + queue
         finally:
             # Scan next directory in the queue
             try:
                 path = queue.pop(0)
             except IndexError: pass
             else:
-                func = partial(
-                    self.thread_pool.call,
-                    self.thread_pool.OTHER,
-                    partial(os.listdir, path),
-                    partial(self.process_directory_listing, path, queue))
+                func = partial(self.other_queue.put,
+                               (partial(os.listdir, path),
+                                partial(self.process_directory_listing,
+                                        path, queue),
+                                QtCore.Qt.LowEventPriority))
                 QtCore.QTimer.singleShot(delay, func)
 
     def add_directory(self, path):
         path = encode_filename(path)
-        self.thread_pool.call(self.thread_pool.OTHER, partial(os.listdir, path),
-                              partial(self.process_directory_listing, path, []))
+        self.other_queue.put((partial(os.listdir, path),
+                              partial(self.process_directory_listing, path, []),
+                              QtCore.Qt.LowEventPriority))
 
     def get_file_by_id(self, id):
         """Get file by a file ID."""
@@ -364,10 +394,10 @@ class Tagger(QtGui.QApplication):
                           self.config.setting["server_port"],
                           self.browser_integration.port)
 
-    def search(self, text, type):
+    def search(self, text, type, adv=False):
         """Search on the MusicBrainz website."""
         lookup = self.get_file_lookup()
-        getattr(lookup, type + "Search")(text)
+        getattr(lookup, type + "Search")(text, adv)
 
     def lookup(self, metadata):
         """Lookup the metadata on the MusicBrainz website."""
@@ -429,6 +459,7 @@ class Tagger(QtGui.QApplication):
     def remove_files(self, files, from_parent=True):
         """Remove files from the tagger."""
         for file in files:
+            self.analyze_queue.remove(file.filename)
             del self.files[file.filename]
             file.remove(from_parent)
 
@@ -454,8 +485,7 @@ class Tagger(QtGui.QApplication):
             if isinstance(obj, File):
                 files.append(obj)
             elif isinstance(obj, Track):
-                if obj.linked_file:
-                    files.append(obj.linked_file)
+                files.extend(obj.linked_files)
             elif isinstance(obj, Album):
                 self.remove_album(obj)
             elif isinstance(obj, Cluster):
@@ -480,10 +510,10 @@ class Tagger(QtGui.QApplication):
 
         disc = Disc()
         self.set_wait_cursor()
-        self.thread_pool.call(
-            self.thread_pool.OTHER,
+        self.other_queue.put((
             partial(disc.read, encode_filename(device)),
-            partial(self._lookup_disc, disc))
+            partial(self._lookup_disc, disc),
+            QtCore.Qt.LowEventPriority))
 
     def _lookup_puid(self, file, result=None, error=None):
         puid = result
@@ -562,6 +592,17 @@ class Tagger(QtGui.QApplication):
     def num_pending_files(self):
         return len([file for file in self.files.values() if file.state == File.PENDING])
 
+def analyze_thread_run_item(thread, queue, filename):
+    next = partial(queue.ofa._lookup_fingerprint, queue.next, filename)
+    priority = QtCore.Qt.LowEventPriority + 1
+    try:
+        result = queue.ofa._calculate_fingerprint(filename)
+    except:
+        import traceback
+        thread.log.error(traceback.format_exc())
+        thread.to_main(next, priority, error=sys.exc_info()[1])
+    else:
+        thread.to_main(next, priority, result=result)
 
 def help():
     print """Usage: %s [OPTIONS] [FILE] [FILE] ...
