@@ -196,7 +196,7 @@ class MainWindow(QtGui.QMainWindow):
 
         # Windows, KDE and Gnome HID use F1 for this. MacOSX seems to have Cmd-H as a standard
         if sys.platform == "darwin":
-            help_shortcut = "Ctrl+H"
+            help_shortcut = "Ctrl+?"
         else:
             help_shortcut = "F1"
 
@@ -264,11 +264,11 @@ class MainWindow(QtGui.QMainWindow):
         self.search_action = QtGui.QAction(icontheme.lookup('system-search'), _(u"Search"), self)
         self.connect(self.search_action, QtCore.SIGNAL("triggered()"), self.search)
 
-        self.cd_lookup_action = QtGui.QAction(icontheme.lookup('media-optical'), _(u"CD Lookup"), self)
+        self.cd_lookup_action = QtGui.QAction(icontheme.lookup('media-optical'), _(u"&CD Lookup..."), self)
         self.cd_lookup_action.setToolTip(_(u"Lookup CD"))
         self.cd_lookup_action.setStatusTip(_(u"Lookup CD"))
         # TR: Keyboard shortcut for "Lookup CD"
-        self.cd_lookup_action.setShortcut(QtGui.QKeySequence(_("Ctrl+L")))
+        self.cd_lookup_action.setShortcut(QtGui.QKeySequence(_("Ctrl+K")))
         self.connect(self.cd_lookup_action, QtCore.SIGNAL("triggered()"), self.tagger.lookup_cd)
 
         self.analyze_action = QtGui.QAction(icontheme.lookup('picard-analyze'), _(u"&Scan"), self)
@@ -277,18 +277,18 @@ class MainWindow(QtGui.QMainWindow):
         self.analyze_action.setShortcut(QtGui.QKeySequence(_(u"Ctrl+Y")))
         self.connect(self.analyze_action, QtCore.SIGNAL("triggered()"), self.analyze)
 
-        self.cluster_action = QtGui.QAction(icontheme.lookup('picard-cluster'), _(u"Cluster"), self)
+        self.cluster_action = QtGui.QAction(icontheme.lookup('picard-cluster'), _(u"Cl&uster"), self)
         self.cluster_action.setEnabled(False)
         # TR: Keyboard shortcut for "Cluster"
         self.cluster_action.setShortcut(QtGui.QKeySequence(_(u"Ctrl+U")))
         self.connect(self.cluster_action, QtCore.SIGNAL("triggered()"), self.cluster)
 
-        self.autotag_action = QtGui.QAction(icontheme.lookup('picard-auto-tag'), _(u"Lookup"), self)
+        self.autotag_action = QtGui.QAction(icontheme.lookup('picard-auto-tag'), _(u"&Lookup"), self)
         self.autotag_action.setToolTip(_(u"Lookup metadata"))
         self.autotag_action.setStatusTip(_(u"Lookup metadata"))
         self.autotag_action.setEnabled(False)
-        # TR: Keyboard shortcut for "Auto Tag"
-        self.autotag_action.setShortcut(QtGui.QKeySequence(_(u"Ctrl+T")))
+        # TR: Keyboard shortcut for "Lookup"
+        self.autotag_action.setShortcut(QtGui.QKeySequence(_(u"Ctrl+L")))
         self.connect(self.autotag_action, QtCore.SIGNAL("triggered()"), self.autotag)
 
         self.edit_tags_action = QtGui.QAction(icontheme.lookup('picard-edit-tags'), _(u"&Details..."), self)
@@ -311,6 +311,11 @@ class MainWindow(QtGui.QMainWindow):
         self.enable_moving_action.setChecked(self.config.setting["move_files"])
         self.connect(self.enable_moving_action, QtCore.SIGNAL("triggered(bool)"), self.toggle_move_files)
 
+        self.enable_tag_saving_action = QtGui.QAction(_(u"Save &Tags"), self)
+        self.enable_tag_saving_action.setCheckable(True)
+        self.enable_tag_saving_action.setChecked(not self.config.setting["dont_write_tags"])
+        self.connect(self.enable_tag_saving_action, QtCore.SIGNAL("triggered(bool)"), self.toggle_tag_saving)
+
         self.tags_from_filenames_action = QtGui.QAction(_(u"Tags From &File Names..."), self)
         self.connect(self.tags_from_filenames_action, QtCore.SIGNAL("triggered()"), self.open_tags_from_filenames)
 
@@ -323,6 +328,9 @@ class MainWindow(QtGui.QMainWindow):
     def toggle_move_files(self, checked):
         self.config.setting["move_files"] = checked
 
+    def toggle_tag_saving(self, checked):
+        self.config.setting["dont_write_tags"] = not checked
+
     def open_tags_from_filenames(self):
         files = self.tagger.get_files_from_objects(self.selected_objects)
         if not files:
@@ -343,6 +351,9 @@ class MainWindow(QtGui.QMainWindow):
         menu = self.menuBar().addMenu(_(u"&Edit"))
         menu.addAction(self.cut_action)
         menu.addAction(self.paste_action)
+        menu.addSeparator()
+        menu.addAction(self.edit_tags_action)
+        menu.addAction(self.remove_action)
         menu = self.menuBar().addMenu(_(u"&View"))
         menu.addAction(self.show_file_browser_action)
         menu.addAction(self.show_cover_art_action)
@@ -352,9 +363,15 @@ class MainWindow(QtGui.QMainWindow):
         menu = self.menuBar().addMenu(_(u"&Options"))
         menu.addAction(self.enable_renaming_action)
         menu.addAction(self.enable_moving_action)
+        menu.addAction(self.enable_tag_saving_action)
         menu.addSeparator()
         menu.addAction(self.options_action)
         menu = self.menuBar().addMenu(_(u"&Tools"))
+        menu.addAction(self.cd_lookup_action)
+        menu.addAction(self.autotag_action)
+        menu.addAction(self.analyze_action)
+        menu.addAction(self.cluster_action)
+        menu.addSeparator()
         #menu.addAction(self.generate_playlist_action)
         menu.addAction(self.tags_from_filenames_action)
         self.menuBar().addSeparator()
@@ -372,6 +389,9 @@ class MainWindow(QtGui.QMainWindow):
             self.toolbar.setToolButtonStyle(QtCore.Qt.ToolButtonTextUnderIcon)
         else:
             self.toolbar.setToolButtonStyle(QtCore.Qt.ToolButtonIconOnly)
+        
+        self.cd_lookup_action.setEnabled(len(get_cdrom_drives()) > 0)
+
 
     def create_toolbar(self):
         self.toolbar = toolbar = self.addToolBar(_(u"&Toolbar"))
@@ -432,7 +452,7 @@ class MainWindow(QtGui.QMainWindow):
         text = unicode(self.search_edit.text())
         type = unicode(self.search_combo.itemData(
                        self.search_combo.currentIndex()).toString())
-        self.tagger.search(text, type)
+        self.tagger.search(text, type, self.config.setting["use_adv_search_syntax"])
 
     def add_files(self):
         """Add files to the tagger."""
@@ -457,10 +477,32 @@ class MainWindow(QtGui.QMainWindow):
         """Add directory to the tagger."""
         current_directory = self.config.persist["current_directory"] or QtCore.QDir.homePath()
         current_directory = find_existing_path(unicode(current_directory))
-        directory = QtGui.QFileDialog.getExistingDirectory(self, "", current_directory)
-        if directory:
+
+        dir_list = []
+        if not self.config.setting["toolbar_multiselect"]:
+            directory = QtGui.QFileDialog.getExistingDirectory(self, "", current_directory)
+            if directory:
+                dir_list.append(directory)
+        else:
+            # Use a custom file selection dialog to allow the selection of multiple directories
+            file_dialog = QtGui.QFileDialog(self, "", current_directory)
+            file_dialog.setFileMode(QtGui.QFileDialog.DirectoryOnly)
+            tree_view = file_dialog.findChild(QtGui.QTreeView)
+            tree_view.setSelectionMode(QtGui.QAbstractItemView.ExtendedSelection)
+            list_view = file_dialog.findChild(QtGui.QListView, "listView")
+            list_view.setSelectionMode(QtGui.QAbstractItemView.ExtendedSelection)
+
+            if file_dialog.exec_() == QtGui.QDialog.Accepted:
+                dir_list = file_dialog.selectedFiles()
+
+        if len(dir_list) == 1:
+            self.config.persist["current_directory"] = dir_list[0]
+        elif len(dir_list) > 1:
+            (parent, dir) = os.path.split(str(dir_list[0]))
+            self.config.persist["current_directory"] = parent
+            
+        for directory in dir_list:
             directory = unicode(directory)
-            self.config.persist["current_directory"] = directory
             self.tagger.add_directory(directory)
 
     def generate_playlist(self):
@@ -586,15 +628,18 @@ class MainWindow(QtGui.QMainWindow):
                     statusBar += _(" (Error: %s)") % obj.error
                 file = obj
             elif isinstance(obj, Track):
-                if obj.linked_file:
-                    orig_metadata = obj.linked_file.orig_metadata
-                    metadata = obj.linked_file.metadata
-                    statusBar = "%s (%d%%)" % (obj.linked_file.filename, obj.linked_file.similarity * 100)
-                    if obj.linked_file.state == obj.linked_file.ERROR:
-                        statusBar += _(" (Error: %s)") % obj.linked_file.error
-                    file = obj.linked_file
+                if len(obj.linked_files) == 1:
+                    file = obj.linked_files[0]
+                    orig_metadata = file.orig_metadata
+                    metadata = file.metadata
+                    statusBar = "%s (%d%%)" % (file.filename, file.similarity * 100)
+                    if file.state == file.ERROR:
+                        statusBar += _(" (Error: %s)") % file.error
+                elif len(obj.linked_files) == 0:
+                    metadata = obj.metadata
                 else:
                     metadata = obj.metadata
+                    #Show dup zaper
             elif isinstance(obj, (Cluster, Album)):
                 metadata = obj.metadata
                 is_album = True
